"""Python Fuel Prices consts."""

PROP_FUEL_LOCATION_DYNAMIC_BUILD = "build_on_access"
PROP_FUEL_LOCATION_SOURCE = "source"
PROP_FUEL_LOCATION_SOURCE_ID = "source_id"
PROP_FUEL_LOCATION_PREVENT_CACHE_CLEANUP = "prevent_cache_cleanup"

PROP_AREA_LAT = "lat"
PROP_AREA_LONG = "long"
PROP_AREA_RADIUS = "radius"


ANDROID_USER_AGENT = "Dalvik/2.1.0 (Linux; U; Android 13; Pixel 4 XL Build/TQ3A.230705.001.B4)"
DESKTOP_USER_AGENT = "Mozilla/5.0 (X11; Linux x86_64; rv:120.0) Gecko/20100101 Firefox/120.0"
